@@ -10,17 +10,29 @@
 import logging
 import os
 import sys
+from functools import update_wrapper
 
 import click
-import coloredlogs
 from jinja2.exceptions import UndefinedError
-from termcolor import colored
 
+import coloredlogs
 import k8t
 from k8t import cluster, config, environment, project, scaffolding, values
 from k8t.engine import build
-from k8t.templates import analyze, validate
-from k8t.util import MERGE_METHODS, deep_merge, envvalues, load_yaml, makedirs
+from k8t.templates import YamlValidationError, analyze, render, validate
+from k8t.util import (MERGE_METHODS, deep_merge, envvalues, load_cli_value,
+                      load_yaml)
+
+
+def requires_project_directory(func):
+    @click.pass_context
+    def new_func(ctx, *args, **kwargs):
+        if not project.check_directory(kwargs["directory"]):
+            sys.exit("not a valid project: {}".format(kwargs["directory"]))
+
+        return ctx.invoke(func, *args, **kwargs)
+
+    return update_wrapper(new_func, func)
 
 
 @click.group()
@@ -39,7 +51,7 @@ def root(debug, trace):
 
 @root.command(name="license", help="Print software license.")
 def print_license():
-    print(k8t.__license__)
+    click.echo(k8t.__license__)
 
 # pylint: disable=too-many-locals,too-many-arguments
 @root.command(name="validate", help="Validate template files for given context.")
@@ -48,15 +60,14 @@ def print_license():
 @click.option("--value", "cli_values", type=(str, str), multiple=True, metavar="<KEY VALUE>", help="Additional value(s) to include.")
 @click.option("--cluster", "-c", "cname", help="Cluster context to use.")
 @click.option("--environment", "-e", "ename", help="Deployment environment to use.")
+@click.option("--suffix", "-s", "suffixes", default=[".yaml", ".j2", ".jinja2"], help="Filter template files by suffix. Can be used multiple times.", show_default=True)
 @click.argument("directory", type=click.Path(dir_okay=True, file_okay=False, exists=True), default=os.getcwd())
-def cli_validate(method, value_files, cli_values, cname, ename, directory):
-    if not project.check_directory(directory):
-        sys.exit("not a valid project: {}".format(directory))
-
+@requires_project_directory
+def cli_validate(method, value_files, cli_values, cname, ename, suffixes, directory):
     vals = deep_merge(  # pylint: disable=redefined-outer-name
         values.load_all(directory, cname, ename, method),
         *(load_yaml(p) for p in value_files),
-        dict(cli_values),
+        dict(load_cli_value(k, v) for k, v in cli_values),
         envvalues(),
         method=method,
     )
@@ -66,6 +77,9 @@ def cli_validate(method, value_files, cli_values, cname, ename, directory):
 
     templates = eng.list_templates()  # pylint: disable=redefined-outer-name
 
+    if suffixes:
+        templates = [ name for name in templates if os.path.splitext(name)[1] in suffixes ]
+
     all_validated = True
 
     for template_path in templates:
@@ -87,12 +101,12 @@ def cli_validate(method, value_files, cli_values, cname, ename, directory):
         if errors:
             all_validated = False
 
-            print(colored("{}: ✗".format(template_path), "red"))
+            click.secho("{}: ✗".format(template_path), fg="red")
 
             for error in errors:
-                print("- {}".format(error))
+                click.echo("- {}".format(error))
         else:
-            print(colored("{}: ✔".format(template_path), "green"))
+            click.secho("{}: ✔".format(template_path), fg="green")
 
     sys.exit(not all_validated)
 
@@ -103,28 +117,36 @@ def cli_validate(method, value_files, cli_values, cname, ename, directory):
 @click.option("--value", "cli_values", type=(str, str), multiple=True, metavar="<KEY VALUE>", help="Additional value(s) to include.")
 @click.option("--cluster", "-c", "cname", help="Cluster context to use.")
 @click.option("--environment", "-e", "ename", help="Deployment environment to use.")
+@click.option("--suffix", "-s", "suffixes", default=[".yaml", ".j2", ".jinja2"], help="Filter template files by suffix. Can be used multiple times.", show_default=True)
+@click.option("--secret-provider", help="Secret provider override.")
 @click.argument("directory", type=click.Path(dir_okay=True, file_okay=False, exists=True), default=os.getcwd())
-def cli_gen(method, value_files, cli_values, cname, ename, directory):  # pylint: disable=redefined-outer-name,too-many-arguments
-    if not project.check_directory(directory):
-        sys.exit("not a valid project: {}".format(directory))
-
+@requires_project_directory
+def cli_gen(method, value_files, cli_values, cname, ename, suffixes, secret_provider, directory):  # pylint: disable=redefined-outer-name,too-many-arguments
     vals = deep_merge(  # pylint: disable=redefined-outer-name
         values.load_all(directory, cname, ename, method),
         *(load_yaml(p) for p in value_files),
-        dict(cli_values),
+        dict(load_cli_value(k, v) for k, v in cli_values),
         envvalues(),
         method=method,
     )
+
     config.CONFIG = config.load_all(directory, cname, ename, method)
 
+    if secret_provider is not None:
+        config.CONFIG['secrets']['provider'] = secret_provider
+
     eng = build(directory, cname, ename)
 
     templates = eng.list_templates()  # pylint: disable=redefined-outer-name
+
+    if suffixes:
+        templates = [ name for name in templates if os.path.splitext(name)[1] in suffixes ]
+
     validated = True
 
     for template_path in templates:
         if not validate(template_path, vals, eng):
-            print("Failed to validate template {}".format(template_path))
+            click.echo("Failed to validate template {}".format(template_path))
 
             validated = False
 
@@ -133,11 +155,13 @@ def cli_gen(method, value_files, cli_values, cname, ename, directory):  # pylint
 
     try:
         for template_path in templates:
-            print("---")
-            print("# Source: {}".format(template_path))
-            print(eng.get_template(template_path).render(vals))
-    except UndefinedError as err:
-        print(colored("✗ -> {}".format(err), "red"))
+            click.echo("---")
+            click.echo("# Source: {}".format(template_path))
+
+            template_output = render(template_path, vals, eng)
+            click.echo(template_output)
+    except (UndefinedError, YamlValidationError) as err:
+        click.secho("✗ -> {}".format(err), fg="red")
         sys.exit(1)
 
 
@@ -149,61 +173,48 @@ def new():
 @new.command(name="project", help="Create a new project.")
 @click.argument("directory", type=click.Path())
 def new_project(directory):
-    project.new(directory)
+    scaffolding.new_project(directory)
 
 
 @new.command(name="cluster", help="Create a new cluster context.")
 @click.argument("name")
 @click.argument("directory", type=click.Path(exists=True, file_okay=False), default=os.getcwd())
+@requires_project_directory
 def new_cluster(name, directory):
-    if not project.check_directory(directory):
-        sys.exit("not a valid project: {}".format(directory))
-
-    cluster.new(directory, name)
+    scaffolding.new_cluster(directory, name)
 
 
 @new.command(name="environment", help="Create a new environment context.")
 @click.option("--cluster", "-c", "cname")
 @click.argument("name")
 @click.argument("directory", type=click.Path(exists=True, file_okay=False), default=os.getcwd())
+@requires_project_directory
 def new_environment(cname, name, directory):
-    if not project.check_directory(directory):
-        sys.exit("not a valid project: {}".format(directory))
-
     base_path = project.get_base_dir(directory, cname, environment=None)
 
-    environment.new(base_path, name)
+    scaffolding.new_environment(base_path, name)
 
 
-@new.command(name="template")
+@new.command(name="template", help="Create specified kubernetes manifest template.")
 @click.option("--cluster", "-c", "cname", help="Cluster context to use.")
 @click.option("--environment", "-e", "ename", help="Deployment environment to use.")
 @click.option("--name", "-n", help="Template filename.")
 @click.option("--prefix", "-p", help="Prefix for filename.")
 @click.argument("kind", type=click.Choice(sorted(list(scaffolding.list_available_templates()))))
 @click.argument("directory", type=click.Path(exists=True, file_okay=False), default=os.getcwd())
+@requires_project_directory
 def new_template(cname, ename, name, prefix, kind, directory):
-    if not project.check_directory(directory):
-        sys.exit("not a valid project: {}".format(directory))
-
     base_path = project.get_base_dir(directory, cname, ename)
 
-    template_dir = os.path.join(base_path, "templates")
-
-    makedirs(template_dir, warn_exists=False)
-
-    suffix = None if not name else "-{}".format(kind)
-
-    scaffolding.new_template(
-        kind, os.path.join(
-            template_dir,
-            "{0}{1}{2}.yaml.j2".format(
-                prefix or '',
-                name or kind,
-                suffix or ''
-            ))
+    template_name = "{0}{1}{2}{3}.yaml.j2".format(
+        prefix or '',
+        name or '',
+        '-' if name else '',
+        kind
     )
 
+    scaffolding.new_template(base_path, template_name, kind)
+
 
 @root.group(help="Project inspection commands.")
 def get():
@@ -212,37 +223,31 @@ def get():
 
 @get.command(name="clusters", help="Get configured clusters.")
 @click.argument("directory", type=click.Path(exists=True, file_okay=False), default=os.getcwd())
+@requires_project_directory
 def get_clusters(directory):
-    if not project.check_directory(directory):
-        sys.exit("not a valid project: {}".format(directory))
-
     for cluster_path in cluster.list_all(directory):
-        print(cluster_path)
+        click.echo(cluster_path)
 
 
 @get.command(name="environments", help="Get configured environments.")
 @click.option("--cluster", "-c", "cname", help="Cluster context to use.")
 @click.argument("directory", type=click.Path(exists=True, file_okay=False), default=os.getcwd())
+@requires_project_directory
 def get_environments(cname, directory):  # pylint: disable=redefined-outer-name
-    if not project.check_directory(directory):
-        sys.exit("not a valid project: {}".format(directory))
-
     path = project.get_base_dir(directory, cname, environment=None)
 
     for environment_path in environment.list_all(path):
-        print(environment_path)
+        click.echo(environment_path)
 
 
 @get.command(name="templates", help="Get stored templates.")
 @click.option("--cluster", "-c", "cname", help="Cluster context to use.")
 @click.option("--environment", "-e", "ename", help="Deployment environment to use.")
 @click.argument("directory", type=click.Path(exists=True, file_okay=False), default=os.getcwd())
+@requires_project_directory
 def get_templates(directory, cname, ename):  # pylint: disable=redefined-outer-name
-    if not project.check_directory(directory):
-        sys.exit("not a valid project: {}".format(directory))
-
     for template_path in build(directory, cname, ename).list_templates():
-        print(template_path)
+        click.echo(template_path)
 
 
 @root.group(help="Edit local project files.")
@@ -254,10 +259,8 @@ def edit():
 @click.option("--cluster", "-c", "cname", help="Cluster context to use.")
 @click.option("--environment", "-e", "ename", help="Deployment environment to use.")
 @click.argument("directory", type=click.Path(exists=True, file_okay=False), default=os.getcwd())
+@requires_project_directory
 def edit_config(directory, cname, ename):  # pylint: disable=redefined-outer-name
-    if not project.check_directory(directory):
-        sys.exit("not a valid project: {}".format(directory))
-
     file_path: str
 
     if cname is not None:
@@ -278,10 +281,8 @@ def edit_config(directory, cname, ename):  # pylint: disable=redefined-outer-nam
 @click.option("--cluster", "-c", "cname", help="Cluster context to use.")
 @click.option("--environment", "-e", "ename", help="Deployment environment to use.")
 @click.argument("directory", type=click.Path(exists=True, file_okay=False), default=os.getcwd())
+@requires_project_directory
 def edit_values(directory, cname, ename):  # pylint: disable=redefined-outer-name
-    if not project.check_directory(directory):
-        sys.exit("not a valid project: {}".format(directory))
-
     base_dir = project.get_base_dir(directory, cname, ename)
 
     file_path = os.path.join(base_dir, "values.yaml")
