@@ -11,25 +11,15 @@ import os
 from typing import List
 
 from k8t.project import find_files
-from k8t.util import makedirs, touch
 
 
 def list_all(path: str) -> List[str]:
     result: List[str] = []
 
-    for directory in find_files(path, None, None, 'environments', file_ok=False, dir_ok=True):
+    for directory in find_files(path, None, None, 'clusters', file_ok=False, dir_ok=True):
         for _, dirs, _ in os.walk(directory):
             result.extend(dirs)
 
             break
 
     return result
-
-
-def new(root: str, name: str):
-    directory = os.path.join(root, "clusters", name)
-
-    makedirs(directory)
-
-    touch(os.path.join(directory, "values.yaml"))
-    touch(os.path.join(directory, "config.yaml"))
